// File: create_tin_ex2.cpp
// Author: Dr. Yi Zhang (yizhang-geo@zju.edu.cn)
// Date: 2025-05-15

#include "gctl/mesh/tin.h"
#include "gctl/io/dsv_io.h"

using namespace gctl;

int main(int argc, char const *argv[]) try
{
	// read dem grid
	std::vector<dem_point> topo(8000);
	std::ifstream infile("data/mesh/topo_rnd");
	for (int i = 0; i < 8000; i++)
	{
		infile >> topo[i].x >> topo[i].y >> topo[i].elev;
	}
	infile.close();

	std::vector<double> err_records;
	std::vector<vertex3dc*> tin_vert;
	std::vector<tin_triangle*> tin_ele;
	rnd2tin(topo, tin_vert, tin_ele, 1.0, nullptr, &err_records);

    save_tin2gmsh("data/mesh/tin_out", tin_vert, tin_ele);
    save_tin2triangle("data/mesh/tin_out", tin_vert, tin_ele);

	// Write a log file
	std::ofstream logfile("data/mesh/tin_out.log");
	logfile << "Insertion Maxi-Error\n";
	for (int i = 0; i < err_records.size(); ++i)
	{
		logfile << i+1 << " " << err_records[i] << std::endl;
	}
	logfile.close();

	// Destroy memories allocated by the dem2tin function
	destroy_vector(tin_vert);
	destroy_vector(tin_ele);
	return 0;
}
catch (std::exception &e)
{
    GCTL_ShowWhatError(e.what(), GCTL_ERROR_ERROR, 0, 0, 0);
}