// File: create_tin_ex4.cpp
// Author: Dr. Yi Zhang (yizhang-geo@zju.edu.cn)
// Date: 2025-05-15

#include "gctl/mesh/tin.h"

using namespace gctl;

int main(int argc, char const *argv[]) try
{
	// read dem grid
	std::vector<dem_point> topo(8000);
	std::ifstream infile("data/mesh/topo_rnd");
	for (int i = 0; i < 8000; ++i)
	{
		infile >> topo[i].x >> topo[i].y >> topo[i].elev;
	}
	infile.close();

    // Set outline polygon
    std::vector<vertex3dc> valid_area(8);
    valid_area[0].set(point3dc(-5, 500, 0), 0);
    valid_area[1].set(point3dc(58, 365, 0), 1);
    valid_area[2].set(point3dc(314, 158, 0), 2);
    valid_area[3].set(point3dc(681, 22, 0), 3);
    valid_area[4].set(point3dc(942, 105, 0), 4);
    valid_area[5].set(point3dc(1005, 360, 0), 5);
    valid_area[6].set(point3dc(1005, 1005, 0), 6);
    valid_area[7].set(point3dc(-5, 1005, 0), 7);

	std::vector<double> err_records;
	std::vector<vertex3dc*> tin_vert;
	std::vector<tin_triangle*> tin_ele;
	rnd2tin(topo, tin_vert, tin_ele, 1.0, &valid_area, &err_records);

    save_tin2gmsh("data/mesh/tin_out", tin_vert, tin_ele);
	save_tin2triangle("data/mesh/tin_out", tin_vert, tin_ele);

	// Write a log file
	std::ofstream logfile("data/mesh/tin_out.log");
	logfile << "Insertion Maxi-Error\n";
	for (int i = 0; i < err_records.size(); ++i)
	{
		logfile << i+1 << " " << err_records[i] << std::endl;
	}
	logfile.close();

	// Destroy memories allocated by the dem2tin function
	for (int i = 0; i < tin_vert.size(); ++i)
	destroy_vector(tin_vert);
	destroy_vector(tin_ele);
	return 0;
}
catch (std::exception &e)
{
    GCTL_ShowWhatError(e.what(), GCTL_ERROR_ERROR, 0, 0, 0);
}